# -*- coding: utf-8 -*-
"""This code is part of TestEnv extension

.. module:: yodahelpers.testenv.helpers
   :platform: Unix
   :synopsis: TestEnv helpers
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from yodalib.hydratk.extensions.testenv.db_int import DB_INT as db
from yodalib.hydratk.extensions.testenv.rest_int import REST_INT as rest
from yodalib.hydratk.extensions.testenv.soap_int import SOAP_INT as soap


# -*- coding: utf-8 -*-

"""This code is a part of Hydra Toolkit

.. module:: hydratk.extensions.testenv.translation.en
   :platform: Unix
   :synopsis: English language translation for TestEnv extension
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

language = {
    'name': 'English',
    'ISO-639-1': 'en'
}

msg = {
    'te_received_cmd': ["Received command: '{0}'"],
    'te_delete_db': ["Deleting database: '{0}'"],
    'te_create_db': ["Creating database: '{0}'"],
    'te_install_db': ["Installing database from script: '{0}'"],
    'te_db_installed': ["Database installed successfully"],
    'te_unknown_install': ["Unknown installation script"],
    'te_web_unknown_file': ["Unknown file: '{0}'"],
    'te_db_connected': ["Successfully connected to database"],
    'te_db_disconnected': ["Disconnected from database"],
    'te_db_func': ["Calling DB function: '{0}' with params: '{1}'"],
    'te_db_unknown_entity': ["Unknown entity: '{0}' with ID: '{1}'"],
    'te_db_entity_found': ["Found entity: '{0}' with params: '{1}'"],
    'te_db_entity_created': ["Created entity: '{0}' with ID: '{1}'"],
    'te_db_entity_changed': ["Changed entity: '{0}' with ID: '{1}'"],
    'te_db_role_assigned': ["Assigned role to entity: '{0}'"],
    'te_db_role_revoked': ["Revoked role from entity: '{0}'"],
    'te_db_srv_forbidden': ["Service: '{0}' is forbidden for entity: '{1}'"],
    'te_db_srv_assigned': ["Service: '{0}' is already assigned"],
    'te_db_unknown_srv_param': ["Service: '{0}' has not allowed param: '{1}'"],
    'te_db_mandatory_param': ["Param: '{0}' is mandatory"],
    'te_db_param_not_assigned': ["Param: '{0}' id not assigned"],
    'te_rest_request': ["Received REST request: '{0}' with data: '{1}'"],
    'te_rest_func': ["Calling REST function: '{0}' with params: '{1}'"],
    'te_rest_unknown_entity': ["Unknown entity: '{0}' with ID: '{1}'"],
    'te_rest_entity_found': ["Found entity: '{0}' with params: '{1}'"],
    'te_rest_entity_created': ["Created entity: '{0}' with ID: '{1}'"],
    'te_rest_entity_changed': ["Changed entity: '{0}' with ID: '{1}'"],
    'te_rest_role_assigned': ["Role assigned to entity: '{0}'"],
    'te_rest_role_revoked': ["Role revoked from entity: '{0}'"],
    'te_soap_request': ["Received SOAP request: '{0}' with data: '{1}'"],
    'te_soap_func': ["Calling SOAP function: '{0}' with params: '{1}'"],
    'te_soap_unknown_entity': ["Unknown entity: '{0}' with ID: '{1}'"],
    'te_soap_entity_found': ["Found entity: '{0}' with params: '{1}'"],
    'te_soap_entity_created': ["Created entity: '{0}' with ID: '{1}'"],
    'te_soap_entity_changed': ["Changed entity: '{0}' with ID: '{1}'"],
    'te_soap_role_assigned': ["Role assigned to entity: '{0}'"],
    'te_soap_role_revoked': ["Role revoked from entity: '{0}'"]
}



# -*- coding: utf-8 -*-
"""Handles REST operations

.. module:: testenv.rest_handler
   :platform: Unix
   :synopsis: Handles REST operations
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from hydratk.core.masterhead import MasterHead
from hydratk.extensions.testenv.db_handler import DbHandler
from web import OK, NotFound, BadRequest
from simplejson import loads, dumps


class RestHandler(object):
    """Class RestHandler
    """

    _mh = None

    def __init__(self):
        """Class constructor

        Called when the object is initialized    

        Args:
           none

        """

        self._mh = MasterHead.get_head()

    def _get_db(self):
        """Method connect to database     

        Args:
           none

        Returns:
           DB_INT: DB client                 

        """

        db = DbHandler()
        db.connect()
        return db

    def read_customer(self, data):
        """Method handles GET customer           

        Args:
           data (dict) - request input with customer id 

        Returns:
           http: HTTP 200 with customer detail in JSON,
                 HTTP 404 when customer not found,
                 HTTP 400 when param id is missing 

        Example:

        .. code-block:: javascript

           {
             "id": 1,
             "name": "Charlie Bowman",
             "status": "active",
             "segment": 2,
             "birth_no": "700101/0001",
             "reg_no": "123456",
             "tax_no": "CZ123456"
           }         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'read_customer', data),
                      self._mh.fromhere())

        if ('id' in data):

            db = self._get_db()
            customer = db.read_customer(data.id)
            db.disconnect()

            if (customer != None):
                return customer.tojson()
            else:
                return NotFound

        else:
            return BadRequest()

    def create_customer(self, data):
        """Method handles POST customer           

        Args:
           data (json): customer detail  

        Returns:
           http: HTTP 200 with id of created customer,
                 HTTP 400 when customer not created

        Example:

        .. code-block:: javascript

           {
             "name": "Charlie Bowman",
             "status": "active",
             "segment": 2,
             "birth_no": "700101/0001",
             "reg_no": "123456",
             "tax_no": "CZ123456"
           }         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'create_customer', data),
                      self._mh.fromhere())

        doc = loads(data)
        name = doc['name'] if 'name' in doc else None
        status = doc['status'] if 'status' in doc else 'active'
        segment = doc['segment'] if 'segment' in doc else None
        birth_no = doc['birth_no'] if 'birth_no' in doc else None
        reg_no = doc['reg_no'] if 'reg_no' in doc else None
        tax_no = doc['tax_no'] if 'tax_no' in doc else None

        db = self._get_db()
        id = db.create_customer(
            name, segment, status, birth_no, reg_no, tax_no)
        db.disconnect()

        if (id != None):
            return id
        else:
            return BadRequest()

    def change_customer(self, data):
        """Method handles PUT customer           

        Args:
           data (json): customer detail  

        Returns:
           http: HTTP 200 when customer changed,
                 HTTP 400 when customer not changed

        Example:

        .. code-block:: javascript

           {
             "id": 1,
             "name": "Charlie Bowman",
             "status": "active",
             "segment": 2,
             "birth_no": "700101/0001",
             "reg_no": "123456",
             "tax_no": "CZ123456"
           }                    

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'change_customer', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        name = doc['name'] if 'name' in doc else None
        status = doc['status'] if 'status' in doc else None
        segment = doc['segment'] if 'segment' in doc else None
        birth_no = doc['birth_no'] if 'birth_no' in doc else None
        reg_no = doc['reg_no'] if 'reg_no' in doc else None
        tax_no = doc['tax_no'] if 'tax_no' in doc else None

        db = self._get_db()
        res = db.change_customer(
            id, name, status, segment, birth_no, reg_no, tax_no)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def read_payer(self, data):
        """Method handles GET payer           

        Args:
           data (dict): request input with payer id   

        Returns:
           http: HTTP 200 with payer detail in JSON,
                 HTTP 404 when payer not found,
                 HTTP 400 when param id is missing 

        Example:

        .. code-block:: javascript

           {
             "id": 1,
             "name": "Charlie Bowman",
             "status": "active",
             "billcycle": 1,
             "bank_account": "123456/0100",
             "customer": 1
           }         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'read_payer', data),
                      self._mh.fromhere())

        if ('id' in data):

            db = self._get_db()
            payer = db.read_payer(data.id)
            db.disconnect()

            if (payer != None):
                return payer.tojson()
            else:
                return NotFound

        else:
            return BadRequest()

    def create_payer(self, data):
        """Method handles POST payer           

        Args:
           data (json): payer detail  

        Returns:
           http: HTTP 200 with id of created payer,
                 HTTP 400 when payer not created

        Example:

        .. code-block:: javascript

           {
             "name": "Charlie Bowman",
             "status": "active",
             "billcycle": 1,
             "bank_account": "123456/0100",
             "customer": 1
           }                

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'create_payer', data),
                      self._mh.fromhere())

        doc = loads(data)
        name = doc['name'] if 'name' in doc else None
        status = doc['status'] if 'status' in doc else 'active'
        billcycle = doc['billcycle'] if 'billcycle' in doc else None
        bank_account = doc['bank_account'] if 'bank_account' in doc else None
        customer = doc['customer'] if 'customer' in doc else None

        db = self._get_db()
        id = db.create_payer(name, billcycle, customer, status, bank_account)
        db.disconnect()

        if (id != None):
            return id
        else:
            return BadRequest()

    def change_payer(self, data):
        """Method handles PUT payer           

        Args:
           data (json) - payer detail  

        Returns:
           http: HTTP 200 when payer changed,
                 HTTP 400 when payer not changed

        Example:

        .. code-block:: javascript

           {
             "id": 1,
             "name": "Charlie Bowman",
             "status": "active",
             "billcycle": 1,
             "bank_account": "123456/0100",
             "customer": 1
           }                 

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'change_payer', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        name = doc['name'] if 'name' in doc else None
        status = doc['status'] if 'status' in doc else None
        billcycle = doc['billcycle'] if 'billcycle' in doc else None
        bank_account = doc['bank_account'] if 'bank_account' in doc else None
        customer = doc['customer'] if 'customer' in doc else None

        db = self._get_db()
        res = db.change_payer(
            id, name, status, billcycle, bank_account, customer)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def read_subscriber(self, data):
        """Method handles GET subscriber           

        Args:
           data (dict) - request input with subscriber id   

        Returns:
           http: HTTP 200 with subscriber detail in JSON,
                 HTTP 404 when subscriber not found,
                 HTTP 400 when param id is missing 

        Example:

        .. code-block:: javascript

           {
             "id": 1,
             "name": "Charlie Bowman",
             "msisdn": "123456"
             "status": "active",
             "market": 1,
             "tariff": 433,
             "customer": 1,
             "payer": 2
           }         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'read_subscriber', data),
                      self._mh.fromhere())

        if ('id' in data):

            db = self._get_db()
            subscriber = db.read_subscriber(data.id)
            db.disconnect()

            if (subscriber != None):
                return subscriber.tojson()
            else:
                return NotFound

        else:
            return BadRequest()

    def create_subscriber(self, data):
        """Method handles POST subscriber           

        Args:
           data (json) - subscriber detail  

        Returns:
           http: HTTP 200 with id of created subscriber,
                 HTTP 400 when subscriber not created

        Example:

        .. code-block:: javascript

           {
             "name": "Charlie Bowman",
             "msisdn": "123456"
             "status": "active",
             "market": 1,
             "tariff": 433,
             "customer": 1,
             "payer": 2
           }                 

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'create_subscriber', data),
                      self._mh.fromhere())

        doc = loads(data)
        name = doc['name'] if 'name' in doc else None
        msisdn = doc['msisdn'] if 'msisdn' in doc else None
        status = doc['status'] if 'status' in doc else 'active'
        market = doc['market'] if 'market' in doc else None
        tariff = doc['tariff'] if 'tariff' in doc else None
        customer = doc['customer'] if 'customer' in doc else None
        payer = doc['payer'] if 'payer' in doc else None

        db = self._get_db()
        id = db.create_subscriber(
            name, msisdn, market, tariff, customer, payer, status)
        db.disconnect()

        if (id != None):
            return id
        else:
            return BadRequest()

    def change_subscriber(self, data):
        """Method handles PUT subscriber           

        Args:
           data (json): payer detail  

        Returns:
           http: HTTP 200 when subscriber changed,
                 HTTP 400 when subscriber not changed

        Example:

        .. code-block:: javascript

           {
             "id": 1,
             "name": "Charlie Bowman",
             "msisdn": "123456"
             "status": "active",
             "market": 1,
             "tariff": 433,
             "customer": 1,
             "payer": 2
           }                 

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'change_subscriber', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        name = doc['name'] if 'name' in doc else None
        msisdn = doc['msisdn'] if 'msisdn' in doc else None
        status = doc['status'] if 'status' in doc else None
        market = doc['market'] if 'market' in doc else None
        tariff = doc['tariff'] if 'tariff' in doc else None
        customer = doc['customer'] if 'customer' in doc else None
        payer = doc['payer'] if 'payer' in doc else None

        db = self._get_db()
        res = db.change_subscriber(
            id, name, msisdn, status, market, tariff, customer, payer)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def read_contact(self, data):
        """Method handles GET contact           

        Args:
           data (dict): request input with contact id   

        Returns:
           http: HTTP 200 with contact detail in JSON, choice customer|payer|subscriber,
                 HTTP 404 when contact not found,
                 HTTP 400 when param id is missing 

        Example:

        .. code-block:: javascript  

           {
             "id": 1,
             "name": "Charlie Bowman",
             "phone": "123456"
             "email": "aaa@xxx.com",
             "roles": {
               "role": [
                 {
                   "id": 1,
                   "title": "contract",
                   "customer": 1,
                   "payer": 1,
                   "subscriber": 1
                 }  
             ]}
           }               

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'read_contact', data),
                      self._mh.fromhere())

        if ('id' in data):

            db = self._get_db()
            contact = db.read_contact(data.id)
            db.disconnect()

            if (contact != None):
                return contact.tojson()
            else:
                return NotFound

        else:
            return BadRequest()

    def create_contact(self, data):
        """Method handles POST contact           

        Args:
           data (json): contact detail  

        Returns:
           http: HTTP 200 with id of created contact,
                 HTTP 400 when contact not created

        Example:

        .. code-block:: javascript        

           {
             "name": "Charlie Bowman",
             "phone": "123456"
             "email": "aaa@xxx.com"
           }                 

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'create_contact', data),
                      self._mh.fromhere())

        doc = loads(data)
        name = doc['name'] if 'name' in doc else None
        phone = doc['phone'] if 'phone' in doc else None
        email = doc['email'] if 'email' in doc else None

        db = self._get_db()
        id = db.create_contact(name, phone, email)
        db.disconnect()

        if (id != None):
            return id
        else:
            return BadRequest()

    def change_contact(self, data):
        """Method handles PUT customer           

        Args:
           data (json): contact detail  

        Returns:
           http: HTTP 200 when contact changed,
                 HTTP 400 when contact not changed

        Example:

        .. code-block:: javascript        

           {
             "id": 1,
             "name": "Charlie Bowman",
             "phone": "123456"
             "email": "aaa@xxx.com"
           }             

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'change_contact', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        name = doc['name'] if 'name' in doc else None
        phone = doc['phone'] if 'phone' in doc else None
        email = doc['email'] if 'email' in doc else None

        db = self._get_db()
        res = db.change_contact(id, name, phone, email)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def assign_contact_role(self, data):
        """Method handles POST contact/role           

        Args:
           data (json): contact role detail, choice customer|payer|subscriber   

        Returns:
           http: HTTP 200 when contact role assigned,
                 HTTP 400 when customer role not assigned

        Example:

        .. code-block:: javascript

           {
             "id": 1,
             "title": "contract",
             "customer": 1,
             "payer": 1,
             "subscriber": 1
           }         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'assign_contact_role', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        role = doc['title'] if 'title' in doc else None
        customer = doc['customer'] if 'customer' in doc else None
        payer = doc['payer'] if 'payer' in doc else None
        subscriber = doc['subscriber'] if 'subscriber' in doc else None

        db = self._get_db()
        res = db.assign_contact_role(id, role, customer, payer, subscriber)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def revoke_contact_role(self, data):
        """Method handles PUT contact/role           

        Args:
           data (json): contact role detail, choice customer|payer|subscriber  

        Returns:
           http: HTTP 200 when contact role revoked,
                 HTTP 400 when customer role not revoked

        Example:

        .. code-block:: javascript

           {
             "id": 1,
             "title": "contract",
             "customer": 1,
             "payer": 1,
             "subscriber": 1
           }          

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'revoke_contat_role', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        role = doc['title'] if 'title' in doc else None
        customer = doc['customer'] if 'customer' in doc else None
        payer = doc['payer'] if 'payer' in doc else None
        subscriber = doc['subscriber'] if 'subscriber' in doc else None

        db = self._get_db()
        res = db.revoke_contact_role(id, role, customer, payer, subscriber)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def read_address(self, data):
        """Method handles GET address           

        Args:
           data (dict): request input with address id   

        Returns:
           http: HTTP 200 with address detail in JSON, choice contact|customer|payer|subscriber,
                 HTTP 404 when address not found,
                 HTTP 400 when param id is missing

        Example:

        .. code-block:: javascript  

           {
             "id": 1,
             "street": "Tomickova",
             "street_no": "2144/1"
             "city": "Praha",
             "zip": 14800
             "roles": {
               "role": [
                 {
                   "id": 1,
                   "title": "contract",
                   "contact": 1,
                   "customer": 1,
                   "payer": 1,
                   "subscriber": 1
                 }  
             ]}
           }             

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'read_address', data),
                      self._mh.fromhere())

        if ('id' in data):

            db = self._get_db()
            address = db.read_address(data.id)
            db.disconnect()

            if (address != None):
                return address.tojson()
            else:
                return NotFound

        else:
            return BadRequest()

    def create_address(self, data):
        """Method handles POST address           

        Args:
           data (json): address detail  

        Returns:
           http: HTTP 200 with id of created address,
                 HTTP 400 when address not created

        Example:

        .. code-block:: javascript

           {
             "street": "Tomickova",
             "street_no": "2144/1"
             "city": "Praha",
             "zip": 14800
           }                    

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'create_address', data),
                      self._mh.fromhere())

        doc = loads(data)
        street = doc['street'] if 'street' in doc else None
        street_no = doc['street_no'] if 'street_no' in doc else None
        city = doc['city'] if 'city' in doc else None
        zip = doc['zip'] if 'zip' in doc else None

        db = self._get_db()
        id = db.create_address(street, street_no, city, zip)
        db.disconnect()

        if (id != None):
            return id
        else:
            return BadRequest()

    def change_address(self, data):
        """Method handles PUT address           

        Args:
           data (json): address detail  

        Returns:
           http: HTTP 200 when address changed,
                 HTTP 400 when address not changed

        Example:

        .. code-block:: javascript   

           {
             "id": 1,
             "street": "Tomickova",
             "street_no": "2144/1"
             "city": "Praha",
             "zip": 14800
           }            

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'change_address', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        street = doc['street'] if 'street' in doc else None
        street_no = doc['street_no'] if 'street_no' in doc else None
        city = doc['city'] if 'city' in doc else None
        zip = doc['zip'] if 'zip' in doc else None

        db = self._get_db()
        res = db.change_address(id, street, street_no, city, zip)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def assign_address_role(self, data):
        """Method handles POST address/role           

        Args:
           data (json): address role detail  

        Returns:
           http: HTTP 200 when address role assigned,
                 HTTP 400 when address role not assigned

        Example:

        .. code-block:: javascript

          {
             "id": 1,
             "title": "contract",
             "contact": 1,
             "customer": 1,
             "payer": 1,
             "subscriber": 1
           }         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'assign_address_role', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        role = doc['title'] if 'title' in doc else None
        contact = doc['contact'] if 'contact' in doc else None
        customer = doc['customer'] if 'customer' in doc else None
        payer = doc['payer'] if 'payer' in doc else None
        subscriber = doc['subscriber'] if 'subscriber' in doc else None

        db = self._get_db()
        res = db.assign_address_role(
            id, role, contact, customer, payer, subscriber)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def revoke_address_role(self, data):
        """Method handles PUT address/role           

        Args:
           data (json): address role detail  

        Returns:
           http: HTTP 200 when address role revoked,
                 HTTP 400 when address role not revoked

        Example:

        .. code-block:: javascript

          {
             "id": 1,
             "title": "contract",
             "contact": 1,
             "customer": 1,
             "payer": 1,
             "subscriber": 1
           }         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'revoke_address_role', data),
                      self._mh.fromhere())

        doc = loads(data)
        id = doc['id'] if 'id' in doc else None
        role = doc['title'] if 'title' in doc else None
        contact = doc['contact'] if 'contact' in doc else None
        customer = doc['customer'] if 'customer' in doc else None
        payer = doc['payer'] if 'payer' in doc else None
        subscriber = doc['subscriber'] if 'subscriber' in doc else None

        db = self._get_db()
        res = db.revoke_address_role(
            id, role, contact, customer, payer, subscriber)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def read_services(self, data):
        """Method handles GET service           

        Args:
           data (dict): request input entity and service id (all services if empty), choice customer|payer|subscriber, 

        Returns:
           http: HTTP 200 with list of services in JSON,
                 HTTP 404 when service not found,
                 HTTP 400 when no entity is provided 

        Example:

        .. code-block:: javascript

           {
             "service": 
             {
               "service": 
               [
                 {
                   "id": 615,
                   "name": "Telefonni cislo",
                   "status": "active",
                   "params": 
                   {
                     "entry": 
                     [
                       {
                         "key": 121,
                         "value": "12345"
                       }
                     ]
                   } 
                 }  
               ]
             } 
           }         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'read_services', data),
                      self._mh.fromhere())

        customer = data['customer'] if 'customer' in data else None
        payer = data['payer'] if 'payer' in data else None
        subscriber = data['subscriber'] if 'subscriber' in data else None
        service = data['service'] if 'service' in data else None

        if (customer == None and payer == None and subscriber == None):
            return BadRequest()

        db = self._get_db()
        srv_list = db.read_services(customer, payer, subscriber, service)
        db.disconnect()

        if (len(srv_list) > 0):

            root = {}

            services = []
            for service in srv_list:
                el_service = {
                    'id': service.id, 'name': service.name, 'status': service.status}

                el_params = []
                for key, value in service.params.items():
                    el_params.append({'key': key, 'value': value})

                el_service['params'] = {'entry': el_params}
                services.append(el_service)

            root['services'] = {'service': services}

            return dumps(root)

        else:
            return NotFound

    def create_service(self, data):
        """Method handles POST service           

        Args:
           data (json): service operation, choice customer|payer|subscriber  

        Returns:
           http: HTTP 200 when service created,
                 HTTP 400 when service not created           

        Example:

        .. code-block:: javascript

           {
             "customer": 1,
             "payer": 1,
             "subscriber": 1,
             "service": 615,
             "status": "active",
             "params": 
             {
               "entry": 
               [
                 {
                   "key": 121,
                   "value": "12345"
                 }
               ]
             } 
           }             

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'create_service', data),
                      self._mh.fromhere())

        doc = loads(data)
        customer = doc['customer'] if 'customer' in doc else None
        payer = doc['payer'] if 'payer' in doc else None
        subscriber = doc['subscriber'] if 'subscriber' in doc else None
        service = doc['service'] if 'service' in doc else None
        status = doc['status'] if 'status' in doc else 'active'

        params = {}
        for param in doc['params']['entry']:
            params[param['key']] = param['value']

        db = self._get_db()
        res = db.create_service(
            service, customer, payer, subscriber, status, params)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()

    def change_service(self, data):
        """Method handles PUT service           

        Args:
           data (json): service operation, choice customer|payer|subscriber  

        Returns:
           http: HTTP 200 when service changed,
                 HTTP 400 when service not changed

        Example:

        .. code-block:: javascript

           {
             "customer": 1,
             "payer": 1,
             "subscriber": 1,
             "service": 615,
             "status": "active",
             "params": 
             {
               "entry": 
               [
                 {
                   "key": 121,
                   "value": "12345"
                 }
               ]
             } 
           }          

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_request', 'change_service', data),
                      self._mh.fromhere())

        doc = loads(data)
        customer = doc['customer'] if 'customer' in doc else None
        payer = doc['payer'] if 'payer' in doc else None
        subscriber = doc['subscriber'] if 'subscriber' in doc else None
        service = doc['service'] if 'service' in doc else None
        status = doc['status'] if 'status' in doc else None

        params = {}
        for param in doc['params']['entry']:
            params[param['key']] = param['value']

        db = self._get_db()
        res = db.change_service(
            service, customer, payer, subscriber, status, params)
        db.disconnect()

        if (res):
            return OK()
        else:
            return BadRequest()



# -*- coding: utf-8 -*-
"""Extension for test automation exercises

.. module:: testenv.testenv
   :platform: Unix
   :synopsis: Extension for test automation exercises
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from hydratk.core import extension, bootstrapper
from hydratk.extensions.testenv.web_server import Server
import hydratk.lib.system.config as syscfg
from os import path, remove
from sqlite3 import Error, connect

dep_modules = {
    'hydratk': {
        'min-version': '0.5.0',
        'package': 'hydratk'
    },
    'hydratk.lib.network': {
        'min-version': '0.2.0',
        'package': 'hydratk-lib-network'
    },
    'hydratk.extensions.yoda': {
        'min-version': '0.2.2',
        'package': 'hydratk-ext-yoda'
    },
    'web': {
        'min-version': '0.37',
        'package': 'web.py'
    }
}


class Extension(extension.Extension):
    """Class Extension
    """

    def _init_extension(self):
        """Method initializes extension

        Args:
           none

        Returns:
           void

        """

        self._ext_id = 'testenv'
        self._ext_name = 'TestEnv'
        self._ext_version = '0.2.1'
        self._ext_author = 'Petr Rašek <bowman@hydratk.org>, HydraTK team <team@hydratk.org>'
        self._ext_year = '2015-2017'

        if (not self._check_dependencies()):
            exit(0)

    def _check_dependencies(self):
        """Method checks dependent modules

        Args:            
           none

        Returns:
           bool    

        """

        return bootstrapper._check_dependencies(dep_modules, 'hydratk-ext-testenv')

    def _uninstall(self):
        """Method returns additional uninstall data 

        Args:            
           none

        Returns:
           tuple: list (files), list (modules)    

        """

        files = [
            '/usr/share/man/man1/testenv.1',
            '{0}/hydratk/conf.d/hydratk-ext-testenv.conf'.format(syscfg.HTK_ETC_DIR),
            '{0}/hydratk/testenv'.format(syscfg.HTK_VAR_DIR)
        ]

        if (self._mh.cfg['Extensions']['TestEnv']['ext_dir'].format(var_dir=syscfg.HTK_VAR_DIR) != '{0}/hydratk/testenv'.format(syscfg.HTK_VAR_DIR)):
            files.append(self._mh.cfg['Extensions']['TestEnv']['ext_dir'].format(var_dir=syscfg.HTK_VAR_DIR))

        return files, dep_modules

    def _register_actions(self):
        """Method registers command hooks

        Args:
           none

        Returns:
           void

        """

        if (self._mh.cli_cmdopt_profile == 'testenv'):
            self._register_standalone_actions()
        else:
            self._register_htk_actions()

    def _register_htk_actions(self):
        """Method registers command hooks

        Args:  
           none        

        Returns:
           void

        """

        self._mh.match_cli_command('te-install')
        hook = [{'command': 'te-install', 'callback': self.install_db_fc}]
        self._mh.register_command_hook(hook)

        self._mh.match_cli_command('te-run')
        hook = [{'command': 'te-run', 'callback': self.start_fc}]
        self._mh.register_command_hook(hook)

    def _register_standalone_actions(self):
        """Method registers command hooks for standalone mode

        Args:  
           none        

        Returns:
           void

        """

        option_profile = 'testenv'
        help_title = '{h}' + self._ext_name + ' v' + self._ext_version + '{e}'
        cp_string = '{u}' + "(c) " + self._ext_year + \
            " " + self._ext_author + '{e}'
        self._mh.set_cli_appl_title(help_title, cp_string)

        self._mh.match_cli_command('install', option_profile)
        hook = [{'command': 'install', 'callback': self.install_db_fc}]
        self._mh.register_command_hook(hook)

        self._mh.match_cli_command('run', option_profile)
        hook = [{'command': 'run', 'callback': self.start_fc}]
        self._mh.register_command_hook(hook)

        self._mh.match_cli_command('help', option_profile)

        self._mh.match_cli_option(
            ('c', 'config'), True, 'config', False, option_profile)
        self._mh.match_cli_option(
            ('d', 'debug'), True, 'debug', False, option_profile)
        self._mh.match_cli_option(
            ('e', 'debug-channel'), True, 'debug-channel', False, option_profile)
        self._mh.match_cli_option(
            ('l', 'language'), True, 'language', False, option_profile)
        self._mh.match_cli_option(
            ('m', 'run-mode'), True, 'run-mode', False, option_profile)
        self._mh.match_cli_option(
            ('f', 'force'), False, 'force', False, option_profile)
        self._mh.match_cli_option(
            ('i', 'interactive'), False, 'interactive', False, option_profile)
        self._mh.match_cli_option(
            ('h', 'home'), False, 'home', False, option_profile)

    def install_db_fc(self, ext_call=True):
        """Method handles command te-install    

        Args:
           ext_call (bool): external method call   

        Returns:
           void          

        """

        try:

            if (ext_call):
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
                    'te_received_cmd', 'te-install-db'), self._mh.fromhere())

            ext_dir = self._mh.cfg['Extensions']['TestEnv']['ext_dir'].format(var_dir=syscfg.HTK_VAR_DIR)
            db_file = path.join(
                ext_dir, self._mh.cfg['Extensions']['TestEnv']['db_file'])
            if (path.exists(db_file)):
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
                    'te_delete_db', db_file), self._mh.fromhere())
                remove(db_file)

            install_file = path.join(ext_dir, 'install_db.sql')
            if (path.exists(install_file)):

                with open(install_file, 'r') as file:
                    script = file.read()

                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
                    'te_create_db', db_file), self._mh.fromhere())
                with connect(db_file) as conn:
                    self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
                        'te_install_db', install_file), self._mh.fromhere())
                    cur = conn.cursor()
                    cur.executescript(script)
                    conn.commit()
                    self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
                        'te_db_installed'), self._mh.fromhere())
            else:
                self._mh.dmsg('htk_on_extension_error', self._mh._trn.msg(
                    'te_unknown_install'), self._mh.fromhere())

        except Error as ex:
            self._mh.dmsg(
                'htk_on_extension_error', 'error: {0}'.format(ex), self._mh.fromhere())

    def start_fc(self):
        """Method handles command te-run 

        Starts web server and installs database if not installed  

        Args:
           none

        Returns:
           void                  

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
            'te_received_cmd', 'te-start'), self._mh.fromhere())

        db_file = path.join(self._mh.cfg['Extensions']['TestEnv']['ext_dir'].format(var_dir=syscfg.HTK_VAR_DIR),
                            self._mh.cfg['Extensions']['TestEnv']['db_file'])
        if (not path.exists(db_file)):
            self.install_db_fc(False)

        server = Server()
        server._start()



# -*- coding: utf-8 -*-
"""DB interface methods to be used in helpers

.. module:: yodalib.testenv.db_int
   :platform: Unix
   :synopsis: DB interface methods
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from hydratk.extensions.testenv.db_handler import DbHandler


class DB_INT(object):

    _db = None

    def __init__(self):

        self._db = DbHandler()

    def connect(self):

        return self._db.connect()

    def disconnect(self):

        return self._db.disconnect()

    def read_customer(self, id):

        return self._db.read_customer(id)

    def create_customer(self, name, segment, status='active', birth_no=None, reg_no=None, tax_no=None):

        return self._db.create_customer(name, segment, status, birth_no, reg_no, tax_no)

    def change_customer(self, id, name=None, status=None, segment=None, birth_no=None, reg_no=None, tax_no=None):

        return self._db.change_customer(id, name, status, segment, birth_no, reg_no, tax_no)

    def read_payer(self, id):

        return self._db.read_payer(id)

    def create_payer(self, name, billcycle, customer, status='active', bank_account=None):

        return self._db.create_payer(name, billcycle, customer, status, bank_account)

    def change_payer(self, id, name=None, status=None, billcycle=None, bank_account=None, customer=None):

        return self._db.change_payer(id, name, status, billcycle, bank_account, customer)

    def read_subscriber(self, id):

        return self._db.read_subscriber(id)

    def create_subscriber(self, name, msisdn, market, tariff, customer, payer, status='active'):

        return self._db.create_subscriber(name, msisdn, market, tariff, customer, payer, status)

    def change_subscriber(self, id, name=None, msisdn=None, status=None, market=None, tariff=None, customer=None, payer=None):

        return self._db.change_subscriber(id, name, msisdn, status, market, tariff, customer, payer)

    def read_contact(self, id):

        return self._db.read_contact(id)

    def create_contact(self, name, phone=None, email=None):

        return self._db.create_contact(name, phone, email)

    def change_contact(self, id, name=None, phone=None, email=None):

        return self._db.change_contact(id, name, phone, email)

    def assign_contact_role(self, id, role, customer=None, payer=None, subscriber=None):

        return self._db.assign_contact_role(id, role, customer, payer, subscriber)

    def revoke_contact_role(self, id, role, customer=None, payer=None, subscriber=None):

        return self._db.revoke_contact_role(id, role, customer, payer, subscriber)

    def read_address(self, id):

        return self._db.read_address(id)

    def create_address(self, street, street_no, city, zip):

        return self._db.create_address(street, street_no, city, zip)

    def change_address(self, id, street=None, street_no=None, city=None, zip=None):

        return self._db.change_address(id, street, street_no, city, zip)

    def assign_address_role(self, id, role, contact=None, customer=None, payer=None, subscriber=None):

        return self._db.assign_address_role(id, role, contact, customer, payer, subscriber)

    def revoke_address_role(self, id, role, contact=None, customer=None, payer=None, subscriber=None):

        return self._db.revoke_address_role(id, role, contact, customer, payer, subscriber)

    def read_services(self, customer=None, payer=None, subscriber=None, service=None):

        return self._db.read_services(customer, payer, subscriber, service)

    def create_service(self, service, customer=None, payer=None, subscriber=None, status='active', params={}):

        return self._db.create_service(service, customer, payer, subscriber, status, params)

    def change_service(self, service, customer=None, payer=None, subscriber=None, status=None, params={}):

        return self._db.change_service(service, customer, payer, subscriber, status, params)



# -*- coding: utf-8 -*-
"""Handles DB operations

.. module:: testenv.db_handler
   :platform: Unix
   :synopsis: Handles DB operations
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from hydratk.core.masterhead import MasterHead
from hydratk.lib.network.dbi.client import DBClient
from hydratk.extensions.testenv.entities import Customer, Payer, Subscriber, Service
from hydratk.extensions.testenv.entities import Contact, ContactRole, Address, AddressRole
import hydratk.lib.system.config as syscfg
from os import path


class DbHandler(object):
    """Class DbHandler
    """

    _mh = None
    _db_file = None
    _client = None

    def __init__(self):
        """Class constructor

        Called when the object is initialized 

        Args:  
           none 

        """

        self._mh = MasterHead.get_head()
        self._db_file = path.join(self._mh.cfg['Extensions']['TestEnv']['ext_dir'].format(var_dir=syscfg.HTK_VAR_DIR),
                                  self._mh.cfg['Extensions']['TestEnv']['db_file'])
        self._client = DBClient('SQLITE')

    def connect(self):
        """Method connects to database

        Args:
           none            

        Returns:
           bool: result

        """

        result = self._client.connect(self._db_file)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
            'te_db_connected'), self._mh.fromhere())
        return result

    def disconnect(self):
        """Method disconnects from database

        Args:  
           none          

        Returns:
           bool: result

        """

        result = self._client.disconnect()
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
            'te_db_disconnected'), self._mh.fromhere())
        return result

    def read_customer(self, id):
        """Method reads customer

        Args:
           id (int): customer id           

        Returns:
           obj: crm_entities.Customer

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'read_customer', msg),
                      self._mh.fromhere())

        query = 'SELECT a.id, a.name, b.title AS status, a.segment, a.birth_no, a.reg_no, a.tax_no ' + \
                'FROM customer a, lov_status b WHERE a.id = ? AND a.status = b.id'
        res, row = self._client.exec_query(query, [id], True)

        if (row == None):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'customer', id),
                          self._mh.fromhere())
            return None

        customer = Customer(row['id'], row['name'], row['status'], row['segment'],
                            row['birth_no'], row['reg_no'], row['tax_no'])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_found', 'customer', customer),
                      self._mh.fromhere())

        return customer

    def create_customer(self, name, segment, status='active', birth_no=None, reg_no=None, tax_no=None):
        """Method creates customer

        Args:
           name (str): name
           segment (int): segment id, lov_segment.id
           status (str): status, lov_status.title, default active
           birth_no (str): birth number
           reg_no (str): registration number
           tax_no (str): tax identification number      

        Returns:
           int: created customer id

        """

        msg = 'name:{0}, status:{1}, segment:{2}, birth_no:{3}, reg_no:{4}, tax_no:{5}'.format(
            name, status, segment, birth_no, reg_no, tax_no)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'create_customer', msg),
                      self._mh.fromhere())

        query = 'SELECT id FROM lov_status WHERE title = ?'
        res, row = self._client.exec_query(query, [status], True)

        status_id = row['id'] if (row != None) else None

        query = 'INSERT INTO customer (name, status, segment, birth_no, reg_no, tax_no, create_date) ' + \
                'VALUES (?, ?, ?, ?, ?, ?, CURRENT_TIMESTAMP)'
        self._client.exec_query(
            query, [name, status_id, segment, birth_no, reg_no, tax_no])

        res, row = self._client.exec_query(
            'SELECT last_insert_rowid()', fetch_one=True)
        id = row[0]

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|name:{1}|status:{2}|segment:{3}'.format(id, name, status, segment) + \
              u'|birth_no:{0}|reg_no:{1}|tax_no:{2}'.format(
                  birth_no, reg_no, tax_no)

        self._client.exec_query(
            query, ['customer', id, 'create_customer', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_created', 'customer', id),
                      self._mh.fromhere())

        return id

    def change_customer(self, id, name=None, status=None, segment=None, birth_no=None, reg_no=None, tax_no=None):
        """Method changes customer

        Args:
           id (int): customer id
           name (str): name
           status (str): status, lov_status.title
           segment (int): segment id, lov_segment.id           
           birth_no (str): birth number
           reg_no (str): registration number
           tax_no (str): tax identification number      

        Returns: 
           bool: result

        """

        msg = 'id:{0}, name:{1}, status:{2}, segment:{3}, birth_no:{4}, reg_no:{5}, tax_no:{6}'. format(
              id, name, status, segment, birth_no, reg_no, tax_no)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'change_customer', msg),
                      self._mh.fromhere())

        query = 'SELECT COUNT(*) FROM customer WHERE id = ?'
        res, row = self._client.exec_query(query, [id], True)

        if (row[0] == 0):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'customer', id),
                          self._mh.fromhere())
            return False

        query = 'UPDATE customer SET {0}, modify_date = CURRENT_TIMESTAMP WHERE id = ?'

        if (name != None):
            self._client.exec_query(query.format('name = ?'), [name, id])
        if (status != None):
            query2 = 'SELECT id FROM lov_status WHERE title = ?'
            res, row = self._client.exec_query(query2, [status], True)
            status_id = row['id'] if (row != None) else None

            self._client.exec_query(
                query.format('status = ?'), [status_id, id])
        if (segment != None):
            self._client.exec_query(query.format('segment = ?'), [segment, id])
        if (birth_no != None):
            self._client.exec_query(
                query.format('birth_no = ?'), [birth_no, id])
        if (reg_no != None):
            self._client.exec_query(query.format('reg_no = ?'), [reg_no, id])
        if (tax_no != None):
            self._client.exec_query(query.format('tax_no = ?'), [tax_no, id])

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|name:{1}|status:{2}|segment:{3}'.format(id, name, status, segment) + \
              u'|birth_no:{0}|reg_no:{1}|tax_no:{2}'.format(
                  birth_no, reg_no, tax_no)

        res, rows = self._client.exec_query(
            query, ('customer', id, 'change_customer', log))
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_changed', 'customer', id),
                      self._mh.fromhere())

        return res

    def read_payer(self, id):
        """Method reads payer

        Args:
           id (int): payer id          

        Returns:
           obj: crm_entities.Payer

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'read_Payer', msg),
                      self._mh.fromhere())

        query = 'SELECT a.id, a.name, b.title AS status, a.billcycle, a.bank_account, a.customer ' + \
                'FROM payer a, lov_status b WHERE a.id = ? AND a.status = b.id'
        res, row = self._client.exec_query(query, [id], True)

        if (row == None):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'payer', id),
                          self._mh.fromhere())
            return None

        payer = Payer(row['id'], row['name'], row['status'], row['billcycle'],
                      row['customer'], row['bank_account'])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_found', 'payer', payer),
                      self._mh.fromhere())

        return payer

    def create_payer(self, name, billcycle, customer, status='active', bank_account=None):
        """Method creates payer

        Args:
           name (str): name
           billcycle (int): billcycle, lov_billcycle.id
           customer (int): assigned customer id
           status (str): status, lov_status.title, default active
           bank_account (str): banking account            

        Returns:
           int: created payer id

        """

        msg = 'name:{0}, status:{1}, billcycle:{2}, bank_account:{3}, customer:{4}'.format(
              name, status, billcycle, bank_account, customer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'create_payer', msg),
                      self._mh.fromhere())

        query = 'SELECT id FROM lov_status WHERE title = ?'
        res, row = self._client.exec_query(query, [status], True)
        status_id = row['id'] if (row != None) else None

        query = 'INSERT INTO payer (name, status, billcycle, bank_account, customer, create_date) ' + \
                'VALUES (?, ?, ?, ?, ?, CURRENT_TIMESTAMP)'
        self._client.exec_query(
            query, [name, status_id, billcycle, bank_account, customer])

        res, row = self._client.exec_query(
            'SELECT last_insert_rowid()', fetch_one=True)
        id = row[0]

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|name:{1}|status:{2}|billcycle:{3}'.format(id, name, status, billcycle) + \
              u'|bank_account:{0}|customer:{1}'.format(bank_account, customer)

        self._client.exec_query(query, ['payer', id, 'create_payer', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_created', 'payer', id),
                      self._mh.fromhere())

        return id

    def change_payer(self, id, name=None, status=None, billcycle=None, bank_account=None, customer=None):
        """Method changes payer

        Args:
           id (int): payer id
           name (str): name
           status (str): status, lov_status.title
           billcycle (int): billcycle id, lov_billcycle.id
           bank_account (str): banking account 
           customer (int): assigned customer id                    

        Returns:
           bool: result

        """

        msg = 'id:{0}, name:{1}, status:{2}, billcycle:{3}, bank_account:{4}, customer:{5}'.format(
              id, name, status, billcycle, bank_account, customer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'change_payer', msg),
                      self._mh.fromhere())

        query = 'SELECT COUNT(*) FROM payer WHERE id = ?'
        res, row = self._client.exec_query(query, [id], True
                                           )
        if (row[0] == 0):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'payer', id),
                          self._mh.fromhere())
            return False

        query = 'UPDATE payer SET {0}, modify_date = CURRENT_TIMESTAMP WHERE id = ?'

        if (name != None):
            self._client.exec_query(query.format('name = ?'), [name, id])
        if (status != None):
            query2 = 'SELECT id FROM lov_status WHERE title = ?'
            res, row = self._client.exec_query(query2, [status], True)
            status_id = row[0] if (row != None) else None

            self._client.exec_query(
                query.format('status = ?'), [status_id, id])
        if (billcycle != None):
            self._client.exec_query(
                query.format('billcycle = ?'), [billcycle, id])
        if (bank_account != None):
            self._client.exec_query(
                query.format('bank_account = ?'), [bank_account, id])
        if (customer != None):
            self._client.exec_query(
                query.format('customer = ?'), [customer, id])

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|name:{1}|status:{2}|billcycle:{3}'.format(id, name, status, billcycle) + \
              u'|bank_account:{0}|customer:{1}'.format(bank_account, customer)

        res, rows = self._client.exec_query(
            query, ['payer', id, 'change_payer', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_changed', 'payer', id),
                      self._mh.fromhere())

        return res

    def read_subscriber(self, id):
        """Method reads subscriber

        Args:
           id (int): subscriber id           

        Returns:
           obj: crm_entities.Subscriber

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'read_subscriber', msg),
                      self._mh.fromhere())

        query = 'SELECT a.id, a.name, a.msisdn, b.title AS status, a.market, a.tariff, a.customer, a.payer ' + \
                'FROM subscriber a, lov_status b WHERE a.id = ? AND a.status = b.id'
        res, row = self._client.exec_query(query, [id], True)

        if (row == None):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'subscriber', id),
                          self._mh.fromhere())
            return None

        subscriber = Subscriber(row['id'], row['name'], row['msisdn'], row['status'],
                                row['market'], row['tariff'], row['customer'], row['payer'])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_found', 'subscriber', subscriber),
                      self._mh.fromhere())

        return subscriber

    def create_subscriber(self, name, msisdn, market, tariff, customer, payer, status='active'):
        """Method creates subscriber

        Args:
           name (str): name
           msisdn (str): MSISDN
           market (int): market id, lov_market.id
           tariff (int): tariff id, lov_tariff.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           status (str): status, lov_status.title, default active                              

        Returns:
           int: created subscriber id

        """

        msg = 'name:{0}, msisdn:{1}, status:{2}, market:{3}, tariff:{4}, customer:{5}, payer:{6}'.format(
              name, msisdn, status, market, tariff, customer, payer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'create_subscriber', msg),
                      self._mh.fromhere())

        query = 'SELECT id FROM lov_status WHERE title = ?'
        res, row = self._client.exec_query(query, [status], True)
        status_id = row['id'] if (row != None) else None

        query = 'INSERT INTO subscriber (name, msisdn, status, market, tariff, customer, payer, create_date) ' + \
                'VALUES (?, ?, ?, ?, ?, ?, ?, CURRENT_TIMESTAMP)'
        self._client.exec_query(
            query, [name, msisdn, status_id, market, tariff, customer, payer])

        res, row = self._client.exec_query(
            'SELECT last_insert_rowid()', fetch_one=True)
        id = row[0]

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|name:{1}|msisdn:{2}|status:{3}|market:{4}'.format(id, name, msisdn, status, market) + \
              u'|tariff:{0}|customer:{1}|payer:{2}'.format(
                  tariff, customer, payer)

        self._client.exec_query(
            query, ['subscriber', id, 'create_subscriber', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_created', 'subscriber', id),
                      self._mh.fromhere())

        return id

    def change_subscriber(self, id, name=None, msisdn=None, status=None, market=None, tariff=None, customer=None, payer=None):
        """Method changes subscriber

        Args:
           id (int): subscriber id
           name (str): name
           msisdn (str): MSISDN
           status (str): status, lov_status.title           
           market (int): market id, lov_market.id
           tariff (int): tariff id, lov_tariff.id
           customer (int): assigned customer id
           payer (int): assigned payer id                     

        Returns:
           result: bool

        """

        msg = 'id:{0}, name:{1}, msisdn:{2}, status:{3}, market:{4}, tariff:{5}, customer:{6}, payer:{7}'.format(
              id, name, msisdn, status, market, tariff, customer, payer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'change_subscriber', msg),
                      self._mh.fromhere())

        query = 'SELECT COUNT(*) FROM subscriber WHERE id = ?'
        res, row = self._client.exec_query(query, [id], True)

        if (row[0] == 0):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'subscriber', id),
                          self._mh.fromhere())
            return False

        query = 'UPDATE subscriber SET {0}, modify_date = CURRENT_TIMESTAMP WHERE id = ?'

        if (name != None):
            self._client.exec_query(query.format('name = ?'), [name, id])
        if (msisdn != None):
            self._client.exec_query(query.format('msisdn = ?'), [msisdn, id])
        if (status != None):
            query2 = 'SELECT id FROM lov_status WHERE title = ?'
            res, row = self._client.exec_query(query2, [status], True)
            status_id = row[0] if (row != None) else None

            self._client.exec_query(
                query.format('status = ?'), [status_id, id])
        if (market != None):
            self._client.exec_query(query.format('market = ?'), [market, id])
        if (tariff != None):
            self._client.exec_query(query.format('tariff = ?'), [tariff, id])
        if (customer != None):
            self._client.exec_query(
                query.format('customer = ?'), [customer, id])
        if (payer != None):
            self._client.exec_query(query.format('payer = ?'), [payer, id])

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|name:{1}|msisdn:{2}|status:{3}|market:{4}'.format(id, name, msisdn, status, market) + \
              u'|tariff:{0}|customer:{1}|payer:{2}'.format(
                  tariff, customer, payer)

        res, rows = self._client.exec_query(
            query, ['subscriber', id, 'change_subscriber', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_changed', 'subscriber', id),
                      self._mh.fromhere())

        return res

    def read_contact(self, id):
        """Method reads contact

        Args:
           id (int): contact id         

        Returns:
           obj: crm_entities.Contact

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'read_contact', msg),
                      self._mh.fromhere())

        query = 'SELECT a.id, a.name, a.phone, a.email, ' + \
                'c.title, b.customer, b.payer, b.subscriber ' + \
                'FROM contact a LEFT JOIN contact_role b ON a.id = b.contact ' + \
                'LEFT JOIN lov_contact_role c ON b.contact_role = c.id ' + \
                'WHERE a.id = ?'
        res, rows = self._client.exec_query(query, [id])

        if (len(rows) == 0):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'contact', id),
                          self._mh.fromhere())
            return None

        first = True
        id = None
        name = None
        phone = None
        email = None
        roles = []

        for row in rows:

            if (first):

                id = row['id']
                name = row['name']
                phone = row['phone']
                email = row['email']
                first = False

            if (row['title'] != None):
                roles.append(ContactRole(row['id'], row['title'], row['customer'],
                                         row['payer'], row['subscriber']))

        contact = Contact(id, name, phone, email, roles)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_found', 'contact', contact),
                      self._mh.fromhere())

        return contact

    def create_contact(self, name, phone=None, email=None):
        """Method creates contact

        Args:
           name (str): name
           phone (str): phone number
           email (str): email         

        Returns:
           int: created contact id

        """

        msg = 'name:{0}, phone:{1}, email:{2}'.format(name, phone, email)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'create_contact', msg),
                      self._mh.fromhere())

        query = 'INSERT INTO contact (name, phone, email, create_date) ' + \
                'VALUES (?, ?, ?, CURRENT_TIMESTAMP)'
        self._client.exec_query(query, [name, phone, email])

        res, row = self._client.exec_query(
            'SELECT last_insert_rowid()', fetch_one=True)
        id = row[0]

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|name:{1}|phone:{2}|email:{3}'.format(
            id, name, phone, email)

        self._client.exec_query(query, ['contact', id, 'create_contact', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_created', 'contact', id),
                      self._mh.fromhere())

        return id

    def change_contact(self, id, name=None, phone=None, email=None):
        """Method changes contact

        Args:
           id (int): contact id
           name (str): name
           phone (str): phone number
           email (str): email         

        Returns:
           bool: result

        """

        msg = 'id:{0}, name:{1}, phone:{2}, email:{3}'.format(
            id, name, phone, email)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'change_contact', msg),
                      self._mh.fromhere())

        query = 'SELECT COUNT(*) FROM contact WHERE id = ?'
        res, row = self._client.exec_query(query, [id], True)

        if (row[0] == 0):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'contact', id),
                          self._mh.fromhere())
            return False

        query = 'UPDATE contact SET {0}, modify_date = CURRENT_TIMESTAMP WHERE id = ?'

        if (name != None):
            self._client.exec_query(query.format('name = ?'), [name, id])
        if (phone != None):
            self._client.exec_query(query.format('phone = ?'), (phone, id))
        if (email != None):
            self._client.exec_query(query.format('email = ?'), (email, id))

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|name:{1}|phone:{2}|email:{3}'.format(
            id, name, phone, email)

        res, row = self._client.exec_query(
            query, ['contact', id, 'change_contact', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_changed', 'contact', id),
                      self._mh.fromhere())

        return res

    def assign_contact_role(self, id, role, customer=None, payer=None, subscriber=None):
        """Method assigns contact role

        Args:
           id (int): contact id
           role (str): role title, lov_contact_role.title
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id     

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, customer:{2}, payer:{3}, subscriber:{4}'.format(
              id, role, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'assign_contact_role', msg),
                      self._mh.fromhere())

        query = 'SELECT id FROM lov_contact_role WHERE title = ?'
        res, row = self._client.exec_query(query, [role], True)

        role_id = row['id'] if (row != None) else None
        if (role_id == None):
            return False

        query = 'INSERT INTO contact_role (contact_role, contact, customer, payer, subscriber, create_date) ' + \
                'VALUES (?, ?, ?, ?, ?, CURRENT_TIMESTAMP)'
        self._client.exec_query(
            query, [role_id, id, customer, payer, subscriber])

        res, row = self._client.exec_query(
            'SELECT last_insert_rowid()', fetch_one=True)
        rec_id = row[0]

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|role:{1}|customer:{2}|payer:{3}'.format(id, role, customer, payer) + \
              u'|subscriber:{0}'.format(subscriber)

        res, row = self._client.exec_query(
            query, ['contact_role', rec_id, 'assign_contact_role', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_role_assigned', 'contact'),
                      self._mh.fromhere())

        return res

    def revoke_contact_role(self, id, role, customer=None, payer=None, subscriber=None):
        """Method revokes contact role

        Args:
           id (int): contact id
           role (str): role title, lov_contact_role.title
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id     

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, customer:{2}, payer:{3}, subscriber:{4}'.format(
              id, role, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'revoke_contact_role', msg),
                      self._mh.fromhere())

        query = 'SELECT id FROM lov_contact_role WHERE title = ?'
        res, row = self._client.exec_query(query, [role], True)

        role_id = row[0] if (row != None) else None
        if (role_id == None):
            return False

        query = 'SELECT id FROM contact_role WHERE contact_role = ? AND contact = ? {0}'
        if (customer != None):
            res, row = self._client.exec_query(
                query.format('AND customer = ?'), [role_id, id, customer], True)
        elif (payer != None):
            res, row = self._client.exec_query(
                query.format('AND payer = ?'), [role_id, id, payer], True)
        elif (subscriber != None):
            res, row = self._client.exec_query(
                query.format('AND subscriber = ?'), [role_id, id, subscriber], True)

        rec_id = row[0] if (row != None) else None
        if (rec_id == None):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
                'te_db_unknown_role'), self._mh.fromhere())
            return False

        query = 'DELETE FROM contact_role where id = ?'
        self._client.exec_query(query, [rec_id])

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|role:{1}|customer:{2}|payer:{3}'.format(id, role, customer, payer) + \
              u'|subscriber:{0}'.format(subscriber)

        res, row = self._client.exec_query(
            query, ['contact_role', rec_id, 'revoke_contact_role', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_role_revoke', 'contact'),
                      self._mh.fromhere())

        return res

    def read_address(self, id):
        """Method reads address

        Args:
           id (int): address id         

        Returns:
           obj: crm_entities.Address 

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'read_address', msg),
                      self._mh.fromhere())

        query = 'SELECT a.id, a.street, a.street_no, a.city, a.zip, ' + \
                'c.title, b.contact, b.customer, b.payer, b.subscriber ' + \
                'FROM address a LEFT JOIN address_role b ON a.id = b.address ' + \
                'LEFT JOIN lov_address_role c ON b.address_role = c.id ' + \
                'WHERE a.id = ?'
        res, rows = self._client.exec_query(query, [id])

        if (len(rows) == 0):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'address', id),
                          self._mh.fromhere())
            return None

        first = True
        id = None
        street = None
        street_no = None
        city = None
        zip = None
        roles = []

        for row in rows:

            if (first):

                id = row['id']
                street = row['street']
                street_no = row['street_no']
                city = row['city']
                zip = row['zip']
                first = False

            if (row['title'] != None):
                roles.append(AddressRole(row['id'], row['title'], row['contact'],
                                         row['customer'], row['payer'], row['subscriber']))

        address = Address(id, street, street_no, city, zip, roles)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_found', 'address', address),
                      self._mh.fromhere())

        return address

    def create_address(self, street, street_no, city, zip):
        """Method creates address

        Args:
           street (str): street
           street_no (str): street number
           city (str): city
           zip (int): zip code       

        Returns:
           int: created address id

        """

        msg = 'street:{0}, street_no:{1}, city:{2}, zip:{3}'.format(
            street, street_no, city, zip)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'create_address', msg),
                      self._mh.fromhere())

        query = 'INSERT INTO address (street, street_no, city, zip, create_date) ' + \
                'VALUES (?, ?, ?, ?, CURRENT_TIMESTAMP)'
        self._client.exec_query(query, [street, street_no, city, zip])

        res, row = self._client.exec_query(
            'SELECT last_insert_rowid()', fetch_one=True)
        id = row[0]

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|street:{1}|street_no:{2}|city:{3}'.format(id, street, street_no, city) + \
              u'|zip:{0}'.format(zip)

        self._client.exec_query(query, ['address', id, 'create_address', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_created', 'address', id),
                      self._mh.fromhere())

        return id

    def change_address(self, id, street=None, street_no=None, city=None, zip=None):
        """Method changes address

        Args:
           id (int): address id
           street (str): street
           street_no (str): street number
           city (str): city
           zip (int): zip code    

        Returns:
           bool: result

        """

        msg = 'id:{0}, street:{1}, street_no:{2}, city:{3}, zip:{4}'.format(
            id, street, street_no, city, zip)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'change_address', msg),
                      self._mh.fromhere())

        query = 'SELECT COUNT(*) FROM address WHERE id = ?'
        res, row = self._client.exec_query(query, [id], True)

        if (row[0] == 0):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'address', id),
                          self._mh.fromhere())
            return False

        query = 'UPDATE address SET {0}, modify_date = CURRENT_TIMESTAMP WHERE id = ?'

        if (street != None):
            self._client.exec_query(query.format('street = ?'), [street, id])
        if (street_no != None):
            self._client.exec_query(
                query.format('street_no = ?'), [street_no, id])
        if (city != None):
            self._client.exec_query(query.format('city = ?'), [city, id])
        if (zip != None):
            self._client.exec_query(query.format('zip = ?'), [zip, id])

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|street:{1}|street_no:{2}|city:{3}'.format(id, street, street_no, city) + \
              u'|zip:{0}'.format(zip)

        res, rows = self._client.exec_query(
            query, ['address', id, 'change_address', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_changed', 'address', id),
                      self._mh.fromhere())

        return res

    def assign_address_role(self, id, role, contact=None, customer=None, payer=None, subscriber=None):
        """Method assigns address role

        Args:
           id (int): address id
           role (str): role title, lov_address_role.title
           contact (int): assigned contact id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id  

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, contact:{2}, customer:{3}, payer:{4}, subscriber:{5}'.format(
              id, role, contact, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'assign_address_role', msg),
                      self._mh.fromhere())

        query = 'SELECT id FROM lov_address_role WHERE title = ?'
        res, row = self._client.exec_query(query, [role], True)

        role_id = row[0] if (row != None) else None
        if (role_id == None):
            return False

        query = 'INSERT INTO address_role (address_role, address, contact, customer, payer, subscriber, create_date) ' + \
                'VALUES (?, ?, ?, ?, ?, ?, CURRENT_TIMESTAMP)'
        self._client.exec_query(
            query, [role_id, id, contact, customer, payer, subscriber])

        res, row = self._client.exec_query(
            'SELECT last_insert_rowid()', fetch_one=True)
        rec_id = row[0]

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|role:{1}|contact:{2}|customer:{3}'.format(id, role, contact, customer) + \
              u'|payer:{0}|subscriber:{1}'.format(payer, subscriber)

        res, row = self._client.exec_query(
            query, ['address_role', rec_id, 'assign_address_role', log])
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_role_assigned', 'address'),
                      self._mh.fromhere())

        return res

    def revoke_address_role(self, id, role, contact=None, customer=None, payer=None, subscriber=None):
        """Method revokes address role

        Args:
           id (int): address id
           role (str): role title, lov_address_role.title
           contact (int): assigned contact id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id  

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, contact:{2}, customer:{3}, payer:{4}, subscriber:{5}'.format(
              id, role, contact, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'revoke_address_role', msg),
                      self._mh.fromhere())

        query = 'SELECT id FROM lov_address_role WHERE title = ?'
        res, row = self._client.exec_query(query, [role], True)

        role_id = row[0] if (row != None) else None
        if (role_id == None):
            return False

        query = 'SELECT id FROM address_role WHERE address_role = ? AND address = ? {0}'
        if (contact != None):
            res, row = self._client.exec_query(
                query.format('AND contact = ?'), [role_id, id, contact], True)
        elif (customer != None):
            res, row = self._client.exec_query(
                query.format('AND customer = ?'), [role_id, id, customer], True)
        elif (payer != None):
            res, row = self._client.exec_query(
                query.format('AND payer = ?'), [role_id, id, payer], True)
        elif (subscriber != None):
            res, row = self._client.exec_query(
                query.format('AND subscriber = ?'), [role_id, id, subscriber], True)

        rec_id = row[0] if (row != None) else None
        if (rec_id == None):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg(
                'te_db_unknown_role'), self._mh.fromhere())
            return False

        query = 'DELETE FROM address_role where id = ?'
        self._client.exec_query(query, [rec_id])

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        log = u'id:{0}|role:{1}|contact:{2}|customer:{3}'.format(id, role, contact, customer) + \
              u'|payer:{0}|subscriber:{1}'.format(payer, subscriber)

        self._client.exec_query(
            query, ('address_role', rec_id, 'revoke_address_role', log))
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_role_revoked', 'address'),
                      self._mh.fromhere())

        return res

    def read_services(self, customer=None, payer=None, subscriber=None, service=None):
        """Method read services

        Args:
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           service (int): service id, lov_service.id, default empty, read all services for entity     

        Returns:
           list: list of crm_entities.Service

        """

        msg = 'customer:{0}, payer:{1}, subscriber:{2}, service:{3}'.format(
              customer, payer, subscriber, service)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'read_services', msg),
                      self._mh.fromhere())

        query = 'SELECT a.service, c.title, d.title AS status, b.param, b.value ' + \
                'FROM service a LEFT JOIN service_params b ON a.id = b.service ' + \
                'JOIN lov_service c ON a.service = c.id JOIN lov_status d ON a.status = d.id ' + \
                'WHERE '

        entity = None
        if (customer != None):
            query += 'a.customer = ?'
            entity = customer
        elif (payer != None):
            query += 'a.payer = ?'
            entity = payer
        elif (subscriber != None):
            query += 'a.subscriber = ?'
            entity = subscriber

        if (service == None):
            query += ''
            res, rows = self._client.exec_query(query, [entity])
        else:
            query += ' AND a.service = ?'
            res, rows = self._client.exec_query(query, [entity, service])

        if (len(rows) > 0):

            curr = None
            services = []
            id = None
            title = None
            status = None
            params = None

            for row in rows:

                id = row['service']

                # store service
                if (curr != None and curr != id):
                    services.append(Service(curr, title, status, params))

                # new service
                if (curr == None or curr != id):
                    curr = id
                    title = row['title']
                    status = row['status']
                    params = {}

                # store param
                if (row['param'] != None):
                    params[row['param']] = row['value']

            # last service
            if (id != None):
                services.append(Service(id, title, status, params))

            for service in services:
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_found', 'service', service),
                              self._mh.fromhere())

            return services

        else:
            return None

    def create_service(self, service, customer=None, payer=None, subscriber=None, status='active', params={}):
        """Method creates service

        Args: 
           service (int): service id, lov_service.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           status (str): status, lov_status.title, default active
           params (dict): key (int), lov_service_param.id, value (str)  

        Returns:
           bool: result

        """

        msg = 'service:{0}, customer:{1}, payer:{2}, subscriber:{3}, status:{4}, params:{5}'.format(
              service, customer, payer, subscriber, status, params)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'create_service', msg),
                      self._mh.fromhere())

        query = 'SELECT customer, payer, subscriber FROM lov_service WHERE id = ?'
        res, row = self._client.exec_query(query, [service], True)

        if (len(row) == 0):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'service', service),
                          self._mh.fromhere())
            return False

        if (customer != None):

            if (row['customer'] != 1):
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_srv_forbidden', service, 'customer'),
                              self._mh.fromhere())
                return False

            query = 'SELECT id FROM service WHERE service = ? AND customer = ?'
            res, rows = self._client.exec_query(query, [service, customer])
            if (len(rows) > 0):
                self._mh.dmsg('htk_on_extension_error', self._mh._trn.msg('te_db_srv_assigned', service),
                              self._mh.fromhere())
                return False

        elif (payer != None):

            if (row['payer'] != 1):
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_srv_forbidden', service, 'payer'),
                              self._mh.fromhere())
                return False

            query = 'SELECT id FROM service WHERE service = ? AND payer = ?'
            res, rows = self._client.exec_query(query, [service, payer])
            if (len(rows) > 0):
                self._mh.dmsg('htk_on_extension_error', self._mh._trn.msg('te_db_srv_assigned', service),
                              self._mh.fromhere())
                return False

        elif (subscriber != None):

            if (row['subscriber'] != 1):
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_srv_forbidden', service, 'subscriber'),
                              self._mh.fromhere())
                return False

            query = 'SELECT id FROM service WHERE service = ? AND subscriber = ?'
            res, rows = self._client.exec_query(query, [service, subscriber])
            if (len(rows) > 0):
                self._mh.dmsg('htk_on_extension_error', self._mh._trn.msg('te_db_srv_assigned', service),
                              self._mh.fromhere())
                return False

        query = 'SELECT id FROM lov_status WHERE title = ?'
        res, row = self._client.exec_query(query, [status], True)

        status_id = row[0] if (row != None) else None

        query = 'INSERT INTO service (service, status, customer, payer, subscriber, create_date) ' + \
                'VALUES (?, ?, ?, ?, ?, CURRENT_TIMESTAMP)'
        self._client.exec_query(
            query, [service, status_id, customer, payer, subscriber])

        res, row = self._client.exec_query(
            'SELECT last_insert_rowid()', fetch_one=True)
        rec_id = row[0]

        log = u'service:{0}|status:{1}|customer:{2}|payer:{3}'.format(service, status, customer, payer) +\
              u'|subscriber:{0}|params#'.format(subscriber)

        if (len(params) > 0):
            for key, value in params.items():

                query = 'SELECT mandatory, default_value from lov_service_param WHERE service = ? AND id = ?'
                res, row = self._client.exec_query(query, [service, key], True)

                if (len(row) == 0):
                    self._mh.dmsg('htk_on_extension_error', self._mh._trn.msg('te_db_unknown_srv_param', service, key),
                                  self._mh.fromhere())
                    return False
                elif (row['mandatory'] == 1 and value == None):
                    self._mh.dmsg('htk_on_extension_error', self._mh._trn.msg('te_db_mandatory_param'), key,
                                  self._mh.fromhere())
                    return False
                elif (row['mandatory'] == 0 and value == None):
                    query = 'INSERT INTO service_params (param, value, service, create_date) ' + \
                            'VALUES (?, ?, ?, CURRENT_TIMESTAMP)'
                    self._client.exec_query(
                        query, [key, row['default_value'], rec_id])
                    log += u'id:{0}|value:{1}'.format(key,
                                                      row['default_value'])
                else:
                    query = 'INSERT INTO service_params (param, value, service, create_date) ' + \
                            'VALUES (?, ?, ?, CURRENT_TIMESTAMP)'
                    self._client.exec_query(query, [key, value, rec_id])
                log += u'id:{0}|value:{1}'.format(key, value)

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        res, row = self._client.exec_query(
            query, ['service', rec_id, 'create_service', log])

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_created', 'service', id),
                      self._mh.fromhere())

        return res

    def change_service(self, service, customer=None, payer=None, subscriber=None, status=None, params={}):
        """Method changes service

        Args: 
           service (int): service id, lov_service.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           status (str): status, lov_status.title, default active
           params (dict): key (int), lov_service_param.id, value (str)  

        Returns:
           bool: result

        """

        msg = 'service:{0}, customer:{1}, payer:{2}, subscriber:{3}, status:{4}, params:{5}'.format(
              service, customer, payer, subscriber, status, params)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_func', 'change_service', msg),
                      self._mh.fromhere())

        id = None

        if (customer != None):

            query = 'SELECT id FROM service WHERE service = ? AND customer = ?'
            res, row = self._client.exec_query(
                query, [service, customer], True)

            if (len(row) == 0):
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'service', id),
                              self._mh.fromhere())
                return False

        elif (payer != None):

            query = 'SELECT id FROM service WHERE service = ? AND payer = ?'
            res, row = self._client.exec_query(query, [service, payer])

            if (len(row) == 0):
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'service', id),
                              self._mh.fromhere())
                return False

        elif (subscriber != None):

            query = 'SELECT id FROM service WHERE service = ? AND subscriber = ?'
            res, row = self._client.exec_query(
                query, [service, subscriber], True)

            if (len(row) == 0):
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_unknown_entity', 'service', id),
                              self._mh.fromhere())
                return False

        id = row[0]

        if (status != None):

            query = 'SELECT id FROM lov_status WHERE title = ?'
            res, row = self._client.exec_query(query, [status], True)

            status_id = row[0] if (row != None) else None

            query = 'UPDATE service SET status = ?, modify_date = CURRENT_TIMESTAMP WHERE id = ?'
            self._client.exec_query(query, [status_id, id])

        log = u'service:{0}|status:{1}|customer:{2}|payer:{3}'.format(service, status, customer, payer) +\
              u'|subscriber:{0}|params#'.format(subscriber)

        if (len(params) > 0):
            for key, value in params.items():

                query = 'SELECT a.id, b.mandatory FROM service_params a, lov_service_param b ' + \
                        'WHERE a.service = ? AND a.param = ? AND a.param = b.id'
                res, row = self._client.exec_query(query, [id, key], True)

                if (len(row) == 0):
                    self._mh.dmsg('htk_on_extension_error', self._mh._trn.msg('te_db_param_not_assigned', key),
                                  self._mh.fromhere())
                    return False
                elif (row['mandatory'] == 1 and value == None):
                    self._mh.dmsg('htk_on_extension_error', self._mh._trn.msg('te_db_mandatory_param', key),
                                  self._mh.fromhere())
                    return False
                else:
                    query = 'UPDATE service_params SET value = ?, modify_date = CURRENT_TIMESTAMP ' + \
                            'WHERE id = ?'
                    self._client.exec_query(query, (value, row['id']))
                log += u'id:{0}|value:{1}'.format(key, value)

        query = 'INSERT INTO history (event_date, table_name, table_id, event, log) ' + \
                'VALUES (CURRENT_TIMESTAMP, ?, ?, ?, ?)'
        self._client.exec_query(query, ['service', id, 'change_service', log])

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_db_entity_changed', 'service', id),
                      self._mh.fromhere())

        return True



# -*- coding: utf-8 -*-
"""SOAP interface methods to be used in helpers

.. module:: testenv.interfaces.soap_int
   :platform: Unix
   :synopsis: SOAP interface methods to be used in helpers
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from hydratk.core.masterhead import MasterHead
from hydratk.lib.network.soap.client import SOAPClient
from hydratk.extensions.testenv.entities import Customer, Payer, Subscriber, Service
from hydratk.extensions.testenv.entities import Contact, ContactRole, Address, AddressRole
from lxml.etree import Element, SubElement, tostring


class SOAP_INT(object):
    """Class SOAP_INT
    """

    _mh = None
    _wsdl = None
    _ns = None

    def __init__(self):
        """Class constructor

        Called when the object is initialized  

        Args:  
           none

        """

        self._mh = MasterHead.get_head()
        ip = self._mh.cfg['Extensions']['TestEnv']['server_ip']
        port = self._mh.cfg['Extensions']['TestEnv']['server_port']
        self._wsdl = 'http://{0}:{1}/ws/crm?wsdl'.format(ip, port)
        self._client = SOAPClient()
        self._client.load_wsdl(self._wsdl)
        self._ns = '{http://hydratk.org/}'

    def is_soap_fault(self, res):
        """Method checks if response is SOAP fault

        Args:
           res (str): response

        Returns:
           bool: result

        """

        allowed = ['int', 'bool', 'customer', 'payer',
                   'subscriber', 'contact', 'address', 'services']
        return (res.__class__.__name__ not in allowed)

    def read_customer(self, id):
        """Method reads customer

        Args:
           id (int): customer id           

        Returns:
           obj: crm_entities.Customer

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'read_customer', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'read_customer')
        SubElement(root, 'id').text = str(id)

        res = self._client.send_request(
            'read_customer', body=root, headers={'SOAPAction': 'read_customer'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:
            birth_no = res.birth_no if (hasattr(res, 'birth_no')) else None
            reg_no = res.reg_no if (hasattr(res, 'reg_no')) else None
            tax_no = res.tax_no if (hasattr(res, 'tax_no')) else None
            customer = Customer(
                res.id, res.name, res.status, res.segment, birth_no, reg_no, tax_no)
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_found', 'customer', customer),
                          self._mh.fromhere())
            return customer

    def create_customer(self, name, segment, status='active', birth_no=None, reg_no=None, tax_no=None):
        """Method creates customer

        Args:
           name (str): name
           segment (int): segment id, lov_segment.id
           status (str): status, lov_status.title, default active
           birth_no (str): birth number
           reg_no (str): registration number
           tax_no (str): tax identification number      

        Returns:
           int: created customer id

        """

        msg = 'name:{0}, status:{1}, segment:{2}, birth_no:{3}, reg_no:{4}, tax_no:{5}'.format(
              name, status, segment, birth_no, reg_no, tax_no)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'create_customer', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'create_customer')
        SubElement(root, 'name').text = name
        SubElement(root, 'status').text = status
        SubElement(root, 'segment').text = str(segment)
        if (birth_no != None):
            SubElement(root, 'birth_no').text = birth_no
        if (reg_no != None):
            SubElement(root, 'reg_no').text = reg_no
        if (tax_no != None):
            SubElement(root, 'tax_no').text = tax_no

        res = self._client.send_request('create_customer', body=tostring(
            root), headers={'SOAPAction': 'create_customer'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:
            id = res
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_created', 'customer', id),
                          self._mh.fromhere())
            return id

    def change_customer(self, id, name=None, status=None, segment=None, birth_no=None, reg_no=None, tax_no=None):
        """Method changes customer

        Args:
           id (int): customer id
           name (str): name
           status (str): status, lov_status.title
           segment (int): segment id, lov_segment.id           
           birth_no (str): birth number
           reg_no (str): registration number
           tax_no (str): tax identification number      

        Returns: 
           bool: result

        """

        msg = 'id:{0}, name:{1}, status:{2}, segment:{3}, birth_no:{4}, reg_no:{5}, tax_no:{6}'. format(
            id, name, status, segment, birth_no, reg_no, tax_no)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'change_customer', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'change_customer')
        SubElement(root, 'id').text = str(id)
        if (name != None):
            SubElement(root, 'name').text = name
        if (status != None):
            SubElement(root, 'status').text = status
        if (segment != None):
            SubElement(root, 'segment').text = str(segment)
        if (birth_no != None):
            SubElement(root, 'birth_no').text = birth_no
        if (reg_no != None):
            SubElement(root, 'reg_no').text = reg_no
        if (tax_no != None):
            SubElement(root, 'tax_no').text = tax_no

        res = self._client.send_request('change_customer', body=tostring(
            root), headers={'SOAPAction': 'change_customer'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_changed', 'customer', id),
                          self._mh.fromhere())
            return True

    def read_payer(self, id):
        """Method reads payer

        Args:
           id (int): payer id          

        Returns:
           obj: crm_entities.Payer

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'read_payer', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'read_payer')
        SubElement(root, 'id').text = str(id)

        res = self._client.send_request(
            'read_payer', body=tostring(root), headers={'SOAPAction': 'read_payer'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:
            bank_account = res.bank_account if (
                hasattr(res, 'bank_account')) else None
            payer = Payer(
                res.id, res.name, res.status, res.billcycle, res.customer, bank_account)
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_found', 'payer', payer),
                          self._mh.fromhere())
            return payer

    def create_payer(self, name, billcycle, customer, status='active', bank_account=None):
        """Method creates payer

        Args:
           name (str): name
           billcycle (int): billcycle, lov_billcycle.id
           customer (int): assigned customer id
           status (str): status, lov_status.title, default active
           bank_account (str): banking account            

        Returns:
           int: created payer id

        """

        msg = 'name:{0}, status:{1}, billcycle:{2}, bank_account:{3}, customer:{4}'.format(
              name, status, billcycle, bank_account, customer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'create_payer', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'create_payer')
        SubElement(root, 'name').text = name
        SubElement(root, 'status').text = status
        SubElement(root, 'billcycle').text = str(billcycle)
        if (bank_account != None):
            SubElement(root, 'bank_account').text = bank_account
        SubElement(root, 'customer').text = str(customer)

        res = self._client.send_request(
            'create_payer', body=tostring(root), headers={'SOAPAction': 'create_payer'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:
            id = res
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_created', 'payer', id),
                          self._mh.fromhere())
            return id

    def change_payer(self, id, name=None, status=None, billcycle=None, bank_account=None, customer=None):
        """Method changes payer

        Args:
           id (int): payer id
           name (str): name
           status (str): status, lov_status.title
           billcycle (int): billcycle id, lov_billcycle.id
           bank_account (str): banking account 
           customer (int): assigned customer id                    

        Returns:
           bool: result

        """

        msg = 'id:{0}, name:{1}, status:{2}, billcycle:{3}, bank_account:{4}, customer:{5}'.format(
              id, name, status, billcycle, bank_account, customer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'change_payer', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'change_payer')

        SubElement(root, 'id').text = str(id)
        if (name != None):
            SubElement(root, 'name').text = name
        if (status != None):
            SubElement(root, 'status').text = status
        if (billcycle != None):
            SubElement(root, 'billcycle').text = str(billcycle)
        if (bank_account != None):
            SubElement(root, 'bank_account').text = bank_account
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)

        res = self._client.send_request(
            'change_payer', body=tostring(root), headers={'SOAPAction': 'change_payer'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_changed', 'payer', id),
                          self._mh.fromhere())
            return True

    def read_subscriber(self, id):
        """Method reads subscriber

        Args:
           id (int): subscriber id           

        Returns:
           obj: crm_entities.Subscriber

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'read_subscriber', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'read_subscriber')
        SubElement(root, 'id').text = str(id)

        res = self._client.send_request('read_subscriber', body=tostring(
            root), headers={'SOAPAction': 'read_subscriber'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:
            subscriber = Subscriber(res.id, res.name, res.msisdn, res.status, res.market, res.tariff,
                                    res.customer, res.payer)
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_found', 'subscriber', subscriber),
                          self._mh.fromhere())
            return subscriber

    def create_subscriber(self, name, msisdn, market, tariff, customer, payer, status='active'):
        """Method creates subscriber

        Args:
           name (str): name
           msisdn (str): MSISDN
           market (int): market id, lov_market.id
           tariff (int): tariff id, lov_tariff.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           status (str): status, lov_status.title, default active                              

        Returns:
           int: created subscriber id

        """

        msg = 'name:{0}, msisdn:{1}, status:{2}, market:{3}, tariff:{4}, customer:{5}, payer:{6}'.format(
              name, msisdn, status, market, tariff, customer, payer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'create_subscriber', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'create_subscriber')
        SubElement(root, 'name').text = name
        SubElement(root, 'msisdn').text = msisdn
        SubElement(root, 'status').text = status
        SubElement(root, 'market').text = str(market)
        SubElement(root, 'tariff').text = str(tariff)
        SubElement(root, 'customer').text = str(customer)
        SubElement(root, 'payer').text = str(payer)

        res = self._client.send_request('create_subscriber', body=tostring(
            root), headers={'SOAPAction': 'create_subscriber'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:
            id = res
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_created', 'subscriber', id),
                          self._mh.fromhere())
            return id

    def change_subscriber(self, id, name=None, msisdn=None, status=None, market=None, tariff=None, customer=None, payer=None):
        """Method changes subscriber

        Args:
           id (int): subscriber id
           name (str): name
           msisdn (str): MSISDN
           status (str): status, lov_status.title           
           market (int): market id, lov_market.id
           tariff (int): tariff id, lov_tariff.id
           customer (int): assigned customer id
           payer (int): assigned payer id                     

        Returns:
           result: bool

        """

        msg = 'id:{0}, name:{1}, msisdn:{2}, status:{3}, market:{4}, tariff:{5}, customer:{6}, payer:{7}'.format(
              id, name, msisdn, status, market, tariff, customer, payer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'change_subscriber', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'change_subscriber')
        SubElement(root, 'id').text = str(id)
        if (name != None):
            SubElement(root, 'name').text = name
        if (msisdn != None):
            SubElement(root, 'msisdn').text = msisdn
        if (status != None):
            SubElement(root, 'status').text = status
        if (market != None):
            SubElement(root, 'market').text = str(market)
        if (tariff != None):
            SubElement(root, 'tariff').text = str(tariff)
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)
        if (payer != None):
            SubElement(root, 'payer').text = str(payer)

        res = self._client.send_request('change_subscriber', body=tostring(
            root), headers={'SOAPAction': 'change_subscriber'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_changed', 'subscriber', id),
                          self._mh.fromhere())
            return True

    def read_contact(self, id):
        """Method reads contact

        Args:
           id (int): contact id         

        Returns:
           obj: crm_entities.Contact

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'read_contact', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'read_contact')
        SubElement(root, 'id').text = str(id)

        res = self._client.send_request(
            'read_contact', body=tostring(root), headers={'SOAPAction': 'read_contact'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:

            roles = []
            if (hasattr(res, 'roles')):
                for role in res.roles[0]:
                    customer = role.customer if (
                        hasattr(role, 'customer')) else None
                    payer = role.payer if (hasattr(role, 'payer')) else None
                    subscriber = role.subscriber if (
                        hasattr(role, 'subscriber')) else None
                    roles.append(
                        ContactRole(role.id, role.title, customer, payer, subscriber))

            phone = res.phone if (hasattr(res, 'phone')) else None
            email = res.email if (hasattr(res, 'email')) else None
            contact = Contact(res.id, res.name, phone, email, roles)
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_found', 'contact', contact),
                          self._mh.fromhere())
            return contact

    def create_contact(self, name, phone=None, email=None):
        """Method creates contact

        Args:
           name (str): name
           phone (str): phone number
           email (str): email         

        Returns:
           int: created contact id

        """

        msg = 'name:{0}, phone:{1}, email:{2}'.format(name, phone, email)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'create_contact', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'create_contact')
        SubElement(root, 'name').text = name
        if (phone != None):
            SubElement(root, 'phone').text = phone
        if (email != None):
            SubElement(root, 'email').text = email

        res = self._client.send_request(
            'create_contact', body=tostring(root), headers={'SOAPAction': 'create_contact'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:
            id = res
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_created', 'contact', id),
                          self._mh.fromhere())
            return id

    def change_contact(self, id, name=None, phone=None, email=None):
        """Method changes contact

        Args:
           id (int): contact id
           name (str): name
           phone (str): phone number
           email (str): email         

        Returns:
           bool: result

        """

        msg = 'id:{0}, name:{1}, phone:{2}, email:{3}'.format(
            id, name, phone, email)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'change_contact', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'change_contact')
        SubElement(root, 'id').text = str(id)
        if (name != None):
            SubElement(root, 'name').text = name
        if (phone != None):
            SubElement(root, 'phone').text = phone
        if (email != None):
            SubElement(root, 'email').text = email

        res = self._client.send_request(
            'change_contact', body=tostring(root), headers={'SOAPAction': 'change_contact'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_changed', 'contact', id),
                          self._mh.fromhere())
            return True

    def assign_contact_role(self, id, role, customer=None, payer=None, subscriber=None):
        """Method assigns contact role

        Args:
           id (int): contact id
           role (str): role title, lov_contact_role.title
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id     

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, customer:{2}, payer:{3}, subscriber:{4}'.format(
              id, role, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'assign_contact_role', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'assign_contact_role')
        SubElement(root, 'id').text = str(id)
        SubElement(root, 'title').text = role
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)
        if (payer != None):
            SubElement(root, 'payer').text = str(payer)
        if (subscriber != None):
            SubElement(root, 'subscriber').text = str(subscriber)

        res = self._client.send_request('assign_contact_role', body=tostring(
            root), headers={'SOAPAction': 'assign_contact_role'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_role_assigned', 'contact'),
                          self._mh.fromhere())
            return True

    def revoke_contact_role(self, id, role, customer=None, payer=None, subscriber=None):
        """Method revokes contact role

        Args:
           id (int): contact id
           role (str): role title, lov_contact_role.title
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id     

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, customer:{2}, payer:{3}, subscriber:{4}'.format(
              id, role, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'revoke_contact_role', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'revoke_contact_role')
        SubElement(root, 'id').text = str(id)
        SubElement(root, 'title').text = role
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)
        if (payer != None):
            SubElement(root, 'payer').text = str(payer)
        if (subscriber != None):
            SubElement(root, 'subscriber').text = str(subscriber)

        res = self._client.send_request('revoke_contact_role', body=tostring(
            root), headers={'SOAPAction': 'revoke_contact_role'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_role_revoked', 'contact'),
                          self._mh.fromhere())
            return True

    def read_address(self, id):
        """Method reads address

        Args:
           id (int): address id         

        Returns:
           obj: crm_entities.Address 

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'read_address', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'read_address')
        SubElement(root, 'id').text = str(id)

        res = self._client.send_request(
            'read_address', body=tostring(root), headers={'SOAPAction': 'read_address'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:

            roles = []
            if (hasattr(res, 'roles')):
                for role in res.roles[0]:
                    contact = role.contact if (
                        hasattr(role, 'contact')) else None
                    customer = role.customer if (
                        hasattr(role, 'customer')) else None
                    payer = role.payer if (hasattr(role, 'payer')) else None
                    subscriber = role.subscriber if (
                        hasattr(role, 'subscriber')) else None
                    roles.append(
                        AddressRole(role.id, role.title, contact, customer, payer, subscriber))

            address = Address(
                res.id, res.street, res.street_no, res.city, res.zip, roles)
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_found', 'address', address),
                          self._mh.fromhere())
            return address

    def create_address(self, street, street_no, city, zip):
        """Method creates address

        Args:
           street (str): street
           street_no (str): street number
           city (str): city
           zip (int): zip code       

        Returns:
           int: created address id

        """

        msg = 'street:{0}, street_no:{1}, city:{2}, zip:{3}'.format(
            street, street_no, city, zip)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'create_address', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'create_address')
        SubElement(root, 'street').text = street
        SubElement(root, 'street_no').text = street_no
        SubElement(root, 'city').text = city
        SubElement(root, 'zip').text = str(zip)

        res = self._client.send_request(
            'create_address', body=tostring(root), headers={'SOAPAction': 'create_address'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return None
        else:
            id = res
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_created', 'address', id),
                          self._mh.fromhere())
            return id

    def change_address(self, id, street=None, street_no=None, city=None, zip=None):
        """Method changes address

        Args:
           id (int): address id
           street (str): street
           street_no (str): street number
           city (str): city
           zip (int): zip code    

        Returns:
           bool: result

        """

        msg = 'id:{0}, street:{1}, street_no:{2}, city:{3}, zip:{4}'.format(
            id, street, street_no, city, zip)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'change_address', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'change_address')
        SubElement(root, 'id').text = str(id)
        if (street != None):
            SubElement(root, 'street').text = street
        if (street_no != None):
            SubElement(root, 'street_no').text = street_no
        if (city != None):
            SubElement(root, 'city').text = city
        if (zip != None):
            SubElement(root, 'zip').text = str(zip)

        res = self._client.send_request(
            'change_address', body=tostring(root), headers={'SOAPAction': 'change_address'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_changed', 'address', id),
                          self._mh.fromhere())
            return True

    def assign_address_role(self, id, role, contact=None, customer=None, payer=None, subscriber=None):
        """Method assigns address role

        Args:
           id (int): address id
           role (str): role title, lov_address_role.title
           contact (int): assigned contact id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id  

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, contact:{2}, customer:{3}, payer:{4}, subscriber:{5}'.format(
              id, role, contact, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'assign_address_role', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'assign_address_role')
        SubElement(root, 'id').text = str(id)
        SubElement(root, 'title').text = role
        if (contact != None):
            SubElement(root, 'contact').text = str(contact)
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)
        if (payer != None):
            SubElement(root, 'payer').text = str(payer)
        if (subscriber != None):
            SubElement(root, 'subscriber').text = str(subscriber)

        res = self._client.send_request('assign_address_role', body=tostring(
            root), headers={'SOAPAction': 'assign_address_role'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_role_assigned', 'address'),
                          self._mh.fromhere())
            return True

    def revoke_address_role(self, id, role, contact=None, customer=None, payer=None, subscriber=None):
        """Method revokes address role

        Args:
           id (int): address id
           role (str): role title, lov_address_role.title
           contact (int): assigned contact id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id  

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, contact:{2}, customer:{3}, payer:{4}, subscriber:{5}'.format(
              id, role, contact, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'revoke_address_role', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'revoke_address_role')
        SubElement(root, 'id').text = str(id)
        SubElement(root, 'title').text = role
        if (contact != None):
            SubElement(root, 'contact').text = str(contact)
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)
        if (payer != None):
            SubElement(root, 'payer').text = str(payer)
        if (subscriber != None):
            SubElement(root, 'subscriber').text = str(subscriber)

        res = self._client.send_request('revoke_address_role', body=tostring(
            root), headers={'SOAPAction': 'revoke_address_role'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_role_revoked', 'address'),
                          self._mh.fromhere())
            return True

    def read_services(self, customer=None, payer=None, subscriber=None, service=None):
        """Method read services

        Args:
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           service (int): service id, lov_service.id, default empty, read all services for entity     

        Returns:
           list: list of crm_entities.Service

        """

        msg = 'customer:{0}, payer:{1}, subscriber:{2}, service:{3}'.format(
              customer, payer, subscriber, service)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'read_services', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'read_services')
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)
        if (payer != None):
            SubElement(root, 'payer').text = str(payer)
        if (subscriber != None):
            SubElement(root, 'subscriber').text = str(subscriber)
        if (service != None):
            SubElement(root, 'service').text = str(service)

        res = self._client.send_request(
            'read_services', body=tostring(root), headers={'SOAPAction': 'read_services'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:

            services = []
            for service in res.service:

                params = {}
                for param in service.params.entry:
                    params[param.key] = param.value

                services.append(
                    Service(service.id, service.name, service.status, params))

            for service in services:
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_found', 'service', service),
                              self._mh.fromhere())

            return services

    def create_service(self, service, customer=None, payer=None, subscriber=None, status='active', params={}):
        """Method creates service

        Args: 
           service (int): service id, lov_service.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           status (str): status, lov_status.title, default active
           params (dict): key (int), lov_service_param.id, value (str)  

        Returns:
           bool: result

        """

        msg = 'service:{0}, customer:{1}, payer:{2}, subscriber:{3}, status:{4}, params:{5}'.format(
              service, customer, payer, subscriber, status, params)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'create_service', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'create_service')
        SubElement(root, 'service').text = str(service)
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)
        if (payer != None):
            SubElement(root, 'payer').text = str(payer)
        if (subscriber != None):
            SubElement(root, 'subscriber').text = str(subscriber)
        SubElement(root, 'status').text = status

        el_params = SubElement(root, 'params')
        for key, value in params.items():
            entry = Element('entry')
            SubElement(entry, 'key').text = str(key)
            SubElement(entry, 'value').text = value
            el_params.append(entry)

        res = self._client.send_request(
            'create_service', body=tostring(root), headers={'SOAPAction': 'create_service'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_created', 'service', service),
                          self._mh.fromhere())
            return True

    def change_service(self, service, customer=None, payer=None, subscriber=None, status=None, params={}):
        """Method changes service

        Args: 
           service (int): service id, lov_service.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           status (str): status, lov_status.title, default active
           params (dict): key (int), lov_service_param.id, value (str)  

        Returns:
           bool: result

        """

        msg = 'service:{0}, customer:{1}, payer:{2}, subscriber:{3}, status:{4}, params:{5}'.format(
              service, customer, payer, subscriber, status, params)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_func', 'change_service', msg),
                      self._mh.fromhere())

        root = Element(self._ns + 'change_service')
        SubElement(root, 'service').text = str(service)
        if (customer != None):
            SubElement(root, 'customer').text = str(customer)
        if (payer != None):
            SubElement(root, 'payer').text = str(payer)
        if (subscriber != None):
            SubElement(root, 'subscriber').text = str(subscriber)
        if (status != None):
            SubElement(root, 'status').text = status

        el_params = SubElement(root, 'params')
        for key, value in params.items():
            entry = Element('entry')
            SubElement(entry, 'key').text = str(key)
            SubElement(entry, 'value').text = value
            el_params.append(entry)

        res = self._client.send_request(
            'change_service', body=tostring(root), headers={'SOAPAction': 'change_service'})

        if (self.is_soap_fault(res)):
            self._mh.dmsg(
                'htk_on_extension_error', 'SOAP fault {0}'.format(res), self._mh.fromhere())
            return False
        else:
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_entity_changed', 'service', service),
                          self._mh.fromhere())
            return True



# -*- coding: utf-8 -*-

"""This code is a part of Hydra Toolkit

.. module:: hydratk.extensions.testenv.translation.cs
   :platform: Unix
   :synopsis: Czech language translation for TestEnv extension
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""
language = {
    'name': 'Čeština',
    'ISO-639-1': 'cs'
}


msg = {
    'te_received_cmd': ["Obdržen příkaz: '{0}'"],
    'te_delete_db': ["Mažu databázi: '{0}'"],
    'te_create_db': ["Vytvářím databázi: '{0}'"],
    'te_install_db': ["Instaluji databázi ze skriptu: '{0}'"],
    'te_db_installed': ["Databáze úspěšně nainstalována"],
    'te_unknown_install': ["Neznámý instalační skript"],
    'te_web_unknown_file': ["Neznámý soubor: '{0}'"],
    'te_db_connected': ["Spojení s databází bylo navázáno"],
    'te_db_disconnected': ["Spojení s databázi bylo ukončeno"],
    'te_db_func': ["Volám DB funkci: '{0}' s parametry: '{1}'"],
    'te_db_unknown_entity': ["Neznámá entita: '{0}' s ID: '{1}'"],
    'te_db_entity_found': ["Nalezena entita: '{0}' s parametry: '{1}'"],
    'te_db_entity_created': ["Vytvořena entita: '{0}' s ID: '{1}'"],
    'te_db_entity_changed': ["Změněna entita: '{0}' s ID: '{1}'"],
    'te_db_role_assigned': ["Přidělěna role k entitě: '{0}'"],
    'te_db_role_revoked': ["Odebrána rola z entity: '{0}'"],
    'te_db_srv_forbidden': ["Služba: '{0}' je zakázána pro entitu: '{1}'"],
    'te_db_srv_assigned': ["Služba: '{0}' je již přidělena"],
    'te_db_unknown_srv_param': ["Služba: '{0}' nemá povolen parametr: '{1}'"],
    'te_db_mandatory_param': ["Parametr: '{0}' je povinný"],
    'te_db_param_not_assigned': ["Parametr: '{0}' není přidělen"],
    'te_rest_request': ["Přijat REST požadavek: '{0}' s daty: '{1}'"],
    'te_rest_func': ["Volám REST funkci: '{0}' s parametry: '{1}'"],
    'te_rest_unknown_entity': ["Neznámá entita: '{0}' s ID: '{1}'"],
    'te_rest_entity_found': ["Nalezena entita: '{0}' s parametry: '{1}'"],
    'te_rest_entity_created': ["Vytvořena entita: '{0}' s ID: '{1}'"],
    'te_rest_entity_changed': ["Změněna entita: '{0}' s ID: '{1}'"],
    'te_rest_role_assigned': ["Přidělěna role k entitě: '{0}'"],
    'te_rest_role_revoked': ["Odebrána rola z entity: '{0}'"],
    'te_soap_request': ["Přijat SOAP požadavek: '{0}' s daty: '{1}'"],
    'te_soap_func': ["Volám SOAP funkci: '{0}' s parametry: '{1}'"],
    'te_soap_unknown_entity': ["Neznámá entita: '{0}' s ID: '{1}'"],
    'te_soap_entity_found': ["Nalezena entita: '{0}' s parametry: '{1}'"],
    'te_soap_entity_created': ["Vytvořena entita: '{0}' s ID: '{1}'"],
    'te_soap_entity_changed': ["Změněna entita: '{0}' s ID: '{1}'"],
    'te_soap_role_assigned': ["Přidělěna role k entitě: '{0}'"],
    'te_soap_role_revoked': ["Odebrána rola z entity: '{0}'"]
}
